@@ -1,6 +1,7 @@
 package clone
 
 import (
+	"context"
 	"fmt"
 	"net/http"
 	"strings"
@@ -18,17 +19,20 @@ import (
 
 type CloneOptions struct {
 	HttpClient func() (*http.Client, error)
+	GitClient  *git.Client
 	Config     func() (config.Config, error)
 	IO         *iostreams.IOStreams
 
-	GitArgs    []string
-	Repository string
+	GitArgs      []string
+	Repository   string
+	UpstreamName string
 }
 
 func NewCmdClone(f *cmdutil.Factory, runF func(*CloneOptions) error) *cobra.Command {
 	opts := &CloneOptions{
 		IO:         f.IOStreams,
 		HttpClient: f.HttpClient,
+		GitClient:  f.GitClient,
 		Config:     f.Config,
 	}
 
@@ -46,7 +50,9 @@ func NewCmdClone(f *cmdutil.Factory, runF func(*CloneOptions) error) *cobra.Comm
 			defaults to the name of the authenticating user.
 
 			If the repository is a fork, its parent repository will be added as an additional
-			git remote called "upstream".
+			git remote called "upstream". The remote name can be configured using %[1]s--upstream-remote-name%[1]s.
+			The %[1]s--upstream-remote-name%[1]s option supports an "@owner" value which will name
+			the remote after the owner of the parent repository.
 		`, "`"),
 		RunE: func(cmd *cobra.Command, args []string) error {
 			opts.Repository = args[0]
@@ -60,6 +66,7 @@ func NewCmdClone(f *cmdutil.Factory, runF func(*CloneOptions) error) *cobra.Comm
 		},
 	}
 
+	cmd.Flags().StringVarP(&opts.UpstreamName, "upstream-remote-name", "u", "upstream", "Upstream remote name when cloning a fork")
 	cmd.SetFlagErrorFunc(func(cmd *cobra.Command, err error) error {
 		if err == pflag.ErrHelp {
 			return err
@@ -107,10 +114,7 @@ func cloneRun(opts *CloneOptions) error {
 		if repositoryIsFullName {
 			fullName = opts.Repository
 		} else {
-			host, err := cfg.DefaultHost()
-			if err != nil {
-				return err
-			}
+			host, _ := cfg.DefaultHost()
 			currentUser, err := api.CurrentLoginName(apiClient, host)
 			if err != nil {
 				return err
@@ -151,7 +155,9 @@ func cloneRun(opts *CloneOptions) error {
 		canonicalCloneURL = strings.TrimSuffix(canonicalCloneURL, ".git") + ".wiki.git"
 	}
 
-	cloneDir, err := git.RunClone(canonicalCloneURL, opts.GitArgs)
+	gitClient := opts.GitClient
+	ctx := context.Background()
+	cloneDir, err := gitClient.Clone(ctx, canonicalCloneURL, opts.GitArgs)
 	if err != nil {
 		return err
 	}
@@ -164,11 +170,15 @@ func cloneRun(opts *CloneOptions) error {
 		}
 		upstreamURL := ghrepo.FormatRemoteURL(canonicalRepo.Parent, protocol)
 
-		err = git.AddUpstreamRemote(upstreamURL, cloneDir, []string{canonicalRepo.Parent.DefaultBranchRef.Name})
+		upstreamName := opts.UpstreamName
+		if opts.UpstreamName == "@owner" {
+			upstreamName = canonicalRepo.Parent.RepoOwner()
+		}
+
+		_, err = gitClient.AddRemote(ctx, upstreamName, upstreamURL, []string{canonicalRepo.Parent.DefaultBranchRef.Name}, git.WithRepoDir(cloneDir))
 		if err != nil {
 			return err
 		}
 	}
-
 	return nil
 }
